from mage_ai.data_preparation.sync import SyncConfig
from mage_ai.data_preparation.sync.base_sync import BaseSync
from mage_ai.shared.logger import VerboseFunctionExec
import git


class GitSync(BaseSync):
    def __init__(self, sync_config: SyncConfig):
        self.remote_repo_link = sync_config.remote_repo_link
        self.repo_path = sync_config.repo_path
        self.branch = sync_config.branch
        try:
            self.repo = git.Repo(self.repo_path)
        except git.exc.InvalidGitRepositoryError:
            self.repo = git.Repo.init(self.repo_path)
            self.repo.create_remote('origin', self.remote_repo_link)

        self.origin = self.repo.remotes.origin

    def sync_data(self):
        with VerboseFunctionExec(
            f'Syncing data with remote repo {self.remote_repo_link}',
            verbose=True,
        ):
            self.origin.fetch(kill_after_timeout=60)
            self.repo.git.reset('--hard', f'origin/{self.branch}')



import os


DEBUG = os.getenv('DEBUG', False)
QUERY_API_KEY = 'api_key'

"""
import secrets
secrets.token_urlsafe()

Make sure this value is the same in mage_ai/frontend/api/constants.ts
"""
OAUTH2_APPLICATION_CLIENT_ID = 'zkWlN0PkIKSN0C11CfUHUj84OT5XOJ6tDZ6bDRO2'
DISABLE_NOTEBOOK_EDIT_ACCESS = os.getenv('DISABLE_NOTEBOOK_EDIT_ACCESS', False)

REQUIRE_USER_AUTHENTICATION = os.getenv('REQUIRE_USER_AUTHENTICATION', False)



from dataclasses import dataclass
from enum import Enum
from mage_ai.shared.config import BaseConfig
import os


class SyncType(str, Enum):
    GIT = 'git'


@dataclass
class SyncConfig(BaseConfig):
    type: SyncType
    remote_repo_link: str
    repo_path: str = os.getcwd()
    branch: str = 'main'
    sync_on_pipeline_run: bool = False



from mage_ai.api.errors import ApiError
from mage_ai.api.resources.GenericResource import GenericResource
from mage_ai.data_preparation.models.file import File
from mage_ai.data_preparation.repo_manager import get_repo_path
from mage_ai.orchestration.db import safe_db_query
import urllib.parse


class FileContentResource(GenericResource):
    @classmethod
    @safe_db_query
    def member(self, pk, user, **kwargs):
        file_path = urllib.parse.unquote(pk)
        file = File.from_path(file_path, get_repo_path())
        return self(file, user, **kwargs)

    def update(self, payload, **kwargs):
        error = ApiError.RESOURCE_INVALID.copy()

        content = payload.get('content')
        if content is None:
            error.update(message='Please provide a \'content\' param in the request payload.')
            raise ApiError(error)

        self.model.update_content(content)
